import requests


def main():
    """
    Define this function to get your results in the form of a dictionary and return it.
    """

    targets = {}

    """
    Your awesome logic goes here

    example:
    targets = requests.get(f"http://10.10.0.1:8081/flagIds").json()
    """

    return targets
