from typing import Iterable
from morel import logger, Targets

log = logger.bind(file=f"{__name__}.log")


def main(target_ip: str) -> Iterable[str] | str:
    # log.debug(f"{__name__} against {target_ip}")

    flags = []

    """
    Your awesome exploit here...
    """

    return flags
    """
    The return value can be a string or any iterable of strings. You can also use yield (but make sure that you fully understand what it does and why yo're using it if you do - in most cases it might result in losing some flags).
    Despite the name you don't have to return strictly flags. The program will search for flags inside the returned value(s) using the provided regex rule.
    """
